'''Classify source packages.

Language packs can be split for different flavours "gnome", "kde", and ""
(which means "all the rest"), so that we can avoid installing a lot of unused
translations on most systems.
'''

import apt_pkg
import gzip
import tempfile
import os

try:
    from urllib.request import urlretrieve
except ImportError:
    # python 2 fallback
    from urllib import urlretrieve

import warnings
warnings.filterwarnings('ignore', '.*apt_pkg.*', DeprecationWarning)

from macros import _file_map


def package_list_map(url, merge_to=None):
    '''Read a package list and parse it into a package->key->value map.

    E. g. package_list_map('Sources.gz')['dpkg']['Version'] would deliver the
    Version of the dpkg source package.
    '''
    # write the uncompressed downloaded list into a temporary file
    downloadfile = urlretrieve(url, tempfile.mkstemp()[1])[0]
    list = tempfile.TemporaryFile()
    try:
        list.write(gzip.open(downloadfile).read())
    except IOError:
        list.write(open(downloadfile).read())
    os.unlink(downloadfile)
    list.seek(0)

    if merge_to is None:
        map = {}
    else:
        map = merge_to
    parser = apt_pkg.TagFile(list)
    while parser.step():
        pkg = {}
        for key in parser.section.keys():
            if key != 'Package':
                pkg[key] = parser.section.get(key)
        map[parser.section.get('Package')] = pkg
    return map


class PackageClassificator:
    def __init__(self, release, domain_map, archive_root):
        '''Classify source packages and domains.

        - release: distro release name (e. g. "trusty")
        - domain_map: file with "srcpackagename domainname" lines
        - archive_root: distro archive URL, for getting Sources.gz
        '''
        # read domain_map.txt
        self.domain_package = {}
        with open(domain_map) as f:
            for line in f:
                f = line.split()
                if len(f) != 2:
                    continue
                self.domain_package[f[1]] = f[0]

        release = release.split('-')[0]

        # get archive map
        self.package_info = package_list_map(
            '%s/dists/%s/main/source/Sources.gz' % (archive_root, release))
        self.package_info = package_list_map(
            '%s/dists/%s/restricted/source/Sources.gz' % (archive_root, release), self.package_info)

    def classify_domain(self, domain):
        '''Classify given domain.

        Raises an exception if the domain is unknown.
        '''
        try:
            pkg = self.domain_package[domain]
        except KeyError:
            raise KeyError('Unknown domain %s' % domain)
        return self.classify_package(pkg)

    def classify_package(self, package):
        '''Classify given source package.

        Raises an exception if the package is unknown.
        '''
        try:
            return _file_map('maps/classify-overrides', package, ':')
        except KeyError:
            pass

        i = self.package_info[package]
        desc = (package + i['Binary'] + i['Section'] + i['Maintainer'] +
                i.get('Build-Depends', '')).lower()
        has_gnome = 'gnome' in desc or 'gtk' in i['Binary'] or package.startswith('gtk')
        has_kde = 'kde' in desc or 'qt4' in desc
        if has_gnome and not has_kde:
            return 'gnome'
        elif has_kde and not has_gnome:
            return 'kde'
        else:
            return ''


# unit tests

if __name__ == '__main__':
    import unittest
    import os.path

    class _T(unittest.TestCase):
        def setUp(self):
            self.rootdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
            dmap = os.path.join(self.rootdir, 'test', 'domain_map.txt')
            self.pc = PackageClassificator('trusty', dmap,
                                           'file://%s/test/archive' % self.rootdir)

        def test_packages(self):
            '''Package classification'''

            self.assertEqual(self.pc.classify_package('gnome-session'), 'gnome')
            self.assertEqual(self.pc.classify_package('gtk+2.0'), 'gnome')
            self.assertEqual(self.pc.classify_package('kdenetwork'), 'kde')
            self.assertEqual(self.pc.classify_package('libdbusmenu-qt'), 'kde')

            # no UI
            self.assertEqual(self.pc.classify_package('postgresql-8.4'), '')

            # both UIs
            self.assertEqual(self.pc.classify_package('jockey'), '')
            self.assertEqual(self.pc.classify_package('unity-2d'), '')

            # unknown
            self.assertRaises(KeyError, self.pc.classify_package, 'unknown')

        def test_domain(self):
            '''Domain classification'''

            self.assertEqual(self.pc.classify_domain('gnome-session-2.0'), 'gnome')
            self.assertEqual(self.pc.classify_domain('gtk+'), 'gnome')
            self.assertEqual(self.pc.classify_domain('knetworkconf'), 'kde')
            self.assertEqual(self.pc.classify_domain('dbusmenu'), 'kde')

            # no UI
            self.assertEqual(self.pc.classify_domain('pg_dump-8.4'), '')

            # both UIs
            self.assertEqual(self.pc.classify_domain('jockey'), '')

            # unknown
            self.assertRaises(KeyError, self.pc.classify_domain, 'unknown')

        def test_overrides(self):
            '''Overrides'''

            # detected as KDE, but works for both
            self.assertEqual(self.pc.classify_package('gdebi'), '')
            self.assertEqual(self.pc.classify_domain('gdebi'), '')

    unittest.main()
